from . import capacity
from . import interesting_periods
from . import perf_attrib
from . import pos
from . import round_trips
from . import timeseries
from . import txn
from . import utils
from .plotting import *  # noqa
from .tears import *  # noqa

try:
    from ._version import version as __version__
    from ._version import version_tuple
except ImportError:
    __version__ = "unknown version"
    version_tuple = (0, 0, "unknown version")

__all__ = [
    "utils",
    "timeseries",
    "pos",
    "txn",
    "interesting_periods",
    "capacity",
    "round_trips",
    "perf_attrib",
]



# -*- coding: utf-8 -*-
import sys
from pathlib import Path
import pydata_sphinx_theme
from pyfolio import __version__ as version

sys.path.insert(0, Path("../..").resolve(strict=True).as_posix())

extensions = [
    "sphinx.ext.autodoc",
    "numpydoc",
    "m2r2",
    "sphinx_markdown_tables",
    "nbsphinx",
    "sphinx.ext.mathjax",
    "sphinx_copybutton",
]

templates_path = ["_templates"]

source_suffix = {".rst": "restructuredtext", ".md": "markdown"}

master_doc = "index"

project = "pyfolio"
copyright = "2016, Quantopian, Inc."
author = "Quantopian, Inc."

release = version
language = None

exclude_patterns = []

highlight_language = "python"

pygments_style = "sphinx"

todo_include_todos = False

html_theme = "pydata_sphinx_theme"
html_theme_path = pydata_sphinx_theme.get_html_theme_path()

html_theme_options = {
    "github_url": "https://github.com/stefan-jansen/pyfolio-reloaded",
    "twitter_url": "https://twitter.com/ml4trading",
    "external_links": [
        {"name": "ML for Trading", "url": "https://ml4trading.io"},
        {"name": "Community", "url": "https://exchange.ml4trading.io"},
    ],
    "google_analytics_id": "UA-74956955-3",
    "use_edit_page_button": True,
    "favicons": [
        {
            "rel": "icon",
            "sizes": "16x16",
            "href": "assets/favicon16x16.ico",
        },
        {
            "rel": "icon",
            "sizes": "32x32",
            "href": "assets/favicon32x32.ico",
        },
    ],
}

html_context = {
    "github_url": "https://github.com",
    "github_user": "stefan-jansen",
    "github_repo": "pyfolio-reloaded",
    "github_version": "main",
    "doc_path": "docs/source",
}

html_static_path = []

htmlhelp_basename = "Pyfoliodoc"

latex_elements = {}

latex_documents = [
    (
        master_doc,
        "Pyfolio.tex",
        "Pyfolio Documentation",
        "Quantopian, Inc.",
        "manual",
    )
]

man_pages = [(master_doc, "pyfolio", "Pyfolio Documentation", [author], 1)]

texinfo_documents = [
    (
        master_doc,
        "Pyfolio",
        "Pyfolio Documentation",
        author,
        "Pyfolio",
        "One line description of project.",
        "Miscellaneous",
    )
]



import functools
from pathlib import Path
import gzip
import inspect
import os
import warnings
from contextlib import contextmanager
from unittest import TestCase

import matplotlib
import matplotlib.pyplot as plt
from pandas import read_csv
from parameterized import parameterized

from pyfolio.tears import (
    create_full_tear_sheet,
    create_simple_tear_sheet,
    create_returns_tear_sheet,
    create_position_tear_sheet,
    create_txn_tear_sheet,
    create_round_trip_tear_sheet,
    create_interesting_times_tear_sheet,
)
from pyfolio.utils import to_utc, to_series


@contextmanager
def _cleanup_cm():
    orig_units_registry = matplotlib.units.registry.copy()
    try:
        with warnings.catch_warnings(), matplotlib.rc_context():
            yield
    finally:
        matplotlib.units.registry.clear()
        matplotlib.units.registry.update(orig_units_registry)
        plt.close("all")


def cleanup(style=None):
    """
    A decorator to ensure that any global state is reset before
    running a test.

    Parameters
    ----------
    style : str, dict, or list, optional
        The style(s) to apply.  Defaults to ``["classic",
        "_classic_test_patch"]``.
    """

    # If cleanup is used without arguments, *style* will be a callable, and we
    # pass it directly to the wrapper generator.  If cleanup if called with an
    # argument, it is a string naming a style, and the function will be passed
    # as an argument to what we return.  This is a confusing, but somewhat
    # standard, pattern for writing a decorator with optional arguments.

    def make_cleanup(func):
        if inspect.isgeneratorfunction(func):

            @functools.wraps(func)
            def wrapped_callable(*args, **kwargs):
                with _cleanup_cm(), matplotlib.style.context(style):
                    yield from func(*args, **kwargs)

        else:

            @functools.wraps(func)
            def wrapped_callable(*args, **kwargs):
                with _cleanup_cm(), matplotlib.style.context(style):
                    func(*args, **kwargs)

        return wrapped_callable

    if callable(style):
        result = make_cleanup(style)
        # Default of mpl_test_settings fixture and image_comparison too.
        style = ["classic", "_classic_test_patch"]
        return result
    else:
        return make_cleanup


class PositionsTestCase(TestCase):
    TEST_DATA = Path(__file__).parent / "test_data"
    # __location__ = os.path.realpath(
    #     os.path.join(os.getcwd(), os.path.dirname(__file__))
    # )

    test_returns = read_csv(
        gzip.open(TEST_DATA / "test_returns.csv.gz"),
        index_col=0,
        parse_dates=True,
    )
    test_returns = to_series(to_utc(test_returns))
    test_txn = to_utc(
        read_csv(
            gzip.open(TEST_DATA / "test_txn.csv.gz"),
            index_col=0,
            parse_dates=True,
        )
    )
    test_pos = to_utc(
        read_csv(
            gzip.open(TEST_DATA / "test_pos.csv.gz"),
            index_col=0,
            parse_dates=True,
        )
    )

    @parameterized.expand(
        [
            ({},),
            ({"slippage": 1},),
            ({"live_start_date": test_returns.index[-20]},),
            ({"round_trips": True},),
            ({"hide_positions": True},),
            ({"cone_std": 1},),
            ({"bootstrap": True},),
        ]
    )
    @cleanup
    def test_create_full_tear_sheet_breakdown(self, kwargs):
        create_full_tear_sheet(
            self.test_returns,
            positions=self.test_pos,
            transactions=self.test_txn,
            benchmark_rets=self.test_returns,
            **kwargs,
        )

    @parameterized.expand(
        [
            ({},),
            ({"slippage": 1},),
            ({"live_start_date": test_returns.index[-20]},),
        ]
    )
    @cleanup
    def test_create_simple_tear_sheet_breakdown(self, kwargs):
        create_simple_tear_sheet(
            self.test_returns,
            positions=self.test_pos,
            transactions=self.test_txn,
            **kwargs,
        )

    @parameterized.expand(
        [
            ({},),
            ({"live_start_date": test_returns.index[-20]},),
            ({"cone_std": 1},),
            ({"bootstrap": True},),
        ]
    )
    @cleanup
    def test_create_returns_tear_sheet_breakdown(self, kwargs):
        create_returns_tear_sheet(
            self.test_returns, benchmark_rets=self.test_returns, **kwargs
        )

    @parameterized.expand(
        [
            ({},),
            ({"hide_positions": True},),
            ({"show_and_plot_top_pos": 0},),
            ({"show_and_plot_top_pos": 1},),
        ]
    )
    @cleanup
    def test_create_position_tear_sheet_breakdown(self, kwargs):
        create_position_tear_sheet(self.test_returns, self.test_pos, **kwargs)

    @parameterized.expand(
        [
            ({},),
            ({"unadjusted_returns": test_returns},),
        ]
    )
    @cleanup
    def test_create_txn_tear_sheet_breakdown(self, kwargs):
        create_txn_tear_sheet(self.test_returns, self.test_pos, self.test_txn, **kwargs)

    @parameterized.expand(
        [
            ({},),
            ({"sector_mappings": {}},),
        ]
    )
    @cleanup
    def test_create_round_trip_tear_sheet_breakdown(self, kwargs):
        create_round_trip_tear_sheet(
            self.test_returns, self.test_pos, self.test_txn, **kwargs
        )

    @parameterized.expand(
        [
            ({},),
            ({"legend_loc": 1},),
        ]
    )
    @cleanup
    def test_create_interesting_times_tear_sheet_breakdown(self, kwargs):
        create_interesting_times_tear_sheet(
            self.test_returns, self.test_returns, **kwargs
        )



#
# Copyright 2016 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Generates a list of historical event dates that may have had
significant impact on markets.  See extract_interesting_date_ranges."""

import datetime as dt
from collections import OrderedDict

import pandas as pd

PERIODS = OrderedDict()
# Dotcom bubble
PERIODS["Dotcom"] = (
    pd.Timestamp("20000310", tzinfo=dt.timezone.utc),
    pd.Timestamp("20000910", tzinfo=dt.timezone.utc),
)

# Lehmann Brothers
PERIODS["Lehman"] = (
    pd.Timestamp("20080801", tzinfo=dt.timezone.utc),
    pd.Timestamp("20081001", tzinfo=dt.timezone.utc),
)

# 9/11
PERIODS["9/11"] = (
    pd.Timestamp("20010911", tzinfo=dt.timezone.utc),
    pd.Timestamp("20011011", tzinfo=dt.timezone.utc),
)

# 05/08/11  US down grade and European Debt Crisis 2011
PERIODS["US downgrade/European Debt Crisis"] = (
    pd.Timestamp("20110805", tzinfo=dt.timezone.utc),
    pd.Timestamp("20110905", tzinfo=dt.timezone.utc),
)

# 16/03/11  Fukushima melt down 2011
PERIODS["Fukushima"] = (
    pd.Timestamp("20110316", tzinfo=dt.timezone.utc),
    pd.Timestamp("20110416", tzinfo=dt.timezone.utc),
)

# 01/08/03  US Housing Bubble 2003
PERIODS["US Housing"] = (
    pd.Timestamp("20030108", tzinfo=dt.timezone.utc),
    pd.Timestamp("20030208", tzinfo=dt.timezone.utc),
)

# 06/09/12  EZB IR Event 2012
PERIODS["EZB IR Event"] = (
    pd.Timestamp("20120910", tzinfo=dt.timezone.utc),
    pd.Timestamp("20121010", tzinfo=dt.timezone.utc),
)

# August 2007, March and September of 2008, Q1 & Q2 2009,
PERIODS["Aug07"] = (
    pd.Timestamp("20070801", tzinfo=dt.timezone.utc),
    pd.Timestamp("20070901", tzinfo=dt.timezone.utc),
)
PERIODS["Mar08"] = (
    pd.Timestamp("20080301", tzinfo=dt.timezone.utc),
    pd.Timestamp("20080401", tzinfo=dt.timezone.utc),
)
PERIODS["Sept08"] = (
    pd.Timestamp("20080901", tzinfo=dt.timezone.utc),
    pd.Timestamp("20081001", tzinfo=dt.timezone.utc),
)
PERIODS["2009Q1"] = (
    pd.Timestamp("20090101", tzinfo=dt.timezone.utc),
    pd.Timestamp("20090301", tzinfo=dt.timezone.utc),
)
PERIODS["2009Q2"] = (
    pd.Timestamp("20090301", tzinfo=dt.timezone.utc),
    pd.Timestamp("20090601", tzinfo=dt.timezone.utc),
)

# Flash Crash (May 6, 2010 + 1 week post),
PERIODS["Flash Crash"] = (
    pd.Timestamp("20100505", tzinfo=dt.timezone.utc),
    pd.Timestamp("20100510", tzinfo=dt.timezone.utc),
)

# April and October 2014).
PERIODS["Apr14"] = (
    pd.Timestamp("20140401", tzinfo=dt.timezone.utc),
    pd.Timestamp("20140501", tzinfo=dt.timezone.utc),
)
PERIODS["Oct14"] = (
    pd.Timestamp("20141001", tzinfo=dt.timezone.utc),
    pd.Timestamp("20141101", tzinfo=dt.timezone.utc),
)

# Market down-turn in August/Sept 2015
PERIODS["Fall2015"] = (
    pd.Timestamp("20150815", tzinfo=dt.timezone.utc),
    pd.Timestamp("20150930", tzinfo=dt.timezone.utc),
)

# Market regimes
PERIODS["Low Volatility Bull Market"] = (
    pd.Timestamp("20050101", tzinfo=dt.timezone.utc),
    pd.Timestamp("20070801", tzinfo=dt.timezone.utc),
)

PERIODS["GFC Crash"] = (
    pd.Timestamp("20070801", tzinfo=dt.timezone.utc),
    pd.Timestamp("20090401", tzinfo=dt.timezone.utc),
)

PERIODS["Recovery"] = (
    pd.Timestamp("20090401", tzinfo=dt.timezone.utc),
    pd.Timestamp("20130101", tzinfo=dt.timezone.utc),
)

PERIODS["New Normal"] = (
    pd.Timestamp("20130101", tzinfo=dt.timezone.utc),
    pd.Timestamp("20180921", tzinfo=dt.timezone.utc),
)

PERIODS["Covid"] = (
    pd.Timestamp("20200211", tzinfo=dt.timezone.utc),
    pd.Timestamp("today", tzinfo=dt.timezone.utc),
)



#!/usr/bin/env python
from __future__ import print_function
from contextlib import contextmanager
from glob import glob
import os
from os.path import basename, exists, isfile
from pathlib import Path
from shutil import move, rmtree
from subprocess import check_call

HERE = Path(__file__).resolve(strict=True).parent
PYFOLIO_ROOT = HERE.parent
TEMP_LOCATION = "/tmp/pyfolio-doc"
TEMP_LOCATION_GLOB = TEMP_LOCATION + "/*"


@contextmanager
def removing(path):
    try:
        yield
    finally:
        rmtree(path)


def ensure_not_exists(path):
    if not exists(path):
        return
    if isfile(path):
        os.unlink(path)
    else:
        rmtree(path)


def main():
    old_dir = Path.cwd()
    print("Moving to %s." % HERE)
    os.chdir(HERE)

    try:
        print("Cleaning docs with 'make clean'")
        check_call(["make", "clean"])
        print("Building docs with 'make html'")
        check_call(["make", "html"])

        print("Clearing temp location '%s'" % TEMP_LOCATION)
        rmtree(TEMP_LOCATION, ignore_errors=True)

        with removing(TEMP_LOCATION):
            print("Copying built files to temp location.")
            move("build/html", TEMP_LOCATION)

            print("Moving to '%s'" % PYFOLIO_ROOT)
            os.chdir(PYFOLIO_ROOT)

            print("Checking out gh-pages branch.")
            check_call(
                [
                    "git",
                    "branch",
                    "-f",
                    "--track",
                    "gh-pages",
                    "origin/gh-pages",
                ]
            )
            check_call(["git", "checkout", "gh-pages"])
            check_call(["git", "reset", "--hard", "origin/gh-pages"])

            print("Copying built files:")
            for file_ in glob(TEMP_LOCATION_GLOB):
                base = basename(file_)

                print("%s -> %s" % (file_, base))
                ensure_not_exists(base)
                move(file_, ".")
    finally:
        os.chdir(old_dir)

    print()
    print("Updated documentation branch in directory %s" % PYFOLIO_ROOT)
    print("If you are happy with these changes, commit and push to gh-pages.")


if __name__ == "__main__":
    main()
